@@ -64,6 +64,10 @@ module MetaBuilder
   # of things, in my humble opinion. The currently recognized such additional
   # parameters are:
   # * :option_parser_short: a short option name for option_parser.
+  # * :namespace: a ruby module that will be searched by #string_to_type
+  #   for a constant. If one of the given name is found, its value is
+  #   returned.
+  # * :shortctus: a hash specifiying strings shortcuts for given values.
   class ParameterType
 
     # A hash that makes the :type value of the _type_ argument correspond
@@ -85,7 +89,7 @@ module MetaBuilder
     # to the ParameterType ancestor. _name_ should be a symbol.
     # Moreover, if the second argument is provided, it automatically
     # creates a #type_name instance method returning this value.
-    def self.type_name(name, public_name = nil)
+    def self.type_name(name, public_name = nil, default_value = nil)
       if @@parameter_types.has_key?(name)
         warn "Redefining type #{name} " +
           "from #{@@parameter_types[name]} to #{self}"
@@ -94,6 +98,9 @@ module MetaBuilder
       self.send(:define_method,:type_name) do
         public_name
       end
+      self.send(:define_method,:default_value) do
+        default_value
+      end
     end      
 
     # This function converts a 'description' of the type wanted
@@ -112,12 +119,19 @@ module MetaBuilder
       end
     end
 
+    # Shortcut to convert directly a string to the given type specification.
+    # Handy shortcut.
+    def self.from_string(type, string)
+      return get_type(type).string_to_type(string)
+    end
+
     # Returns a ParameterType child instance suitable for conversion
     # of the given type specification
     def self.get_type(type)
       return get_param_type(type).new(type)
     end
 
+    
     # Runs the string_to_type conversion hook
     def stt_run_hook(val)
       if @type.key?(:stt_hook)
@@ -133,11 +147,15 @@ module MetaBuilder
     # to define a #string_to_type_internal function rather to redefine
     # #string_to_type
     def string_to_type(string)
-      # First makes a constant lookup.
+      # First, shortcuts:
+      if @type.key?(:shortcuts) and @type[:shortcuts].key? string
+        return stt_run_hook(@type[:shortcuts][string])
+      end
+      # Then, constants lookup.
       if @type.key?(:namespace)
         begin
           return stt_run_hook(lookup_const(string))
-        rescue
+        rescue Exception
         end
       end
       return stt_run_hook(string_to_type_internal(string))
@@ -159,8 +177,14 @@ module MetaBuilder
       return type.to_s
     end
 
+    # Returns a default value for the given type. This is reimplemented
+    # systematically from children, with the ParameterType::type_name
+    # statement.
+    def default_value
+    end
+
     # Looks for the value as a constant specified in the :namespace
-    # modules. Raises InvalidInput if not found.
+    # modules. Raises IncorrectInput if not found.
     def lookup_const(str)
       for mod in [@type[:namespace]].flatten
         begin
@@ -169,7 +193,7 @@ module MetaBuilder
           # Nothing, we still look up
         end
       end
-      raise InvalidInput, "Constant #{str} not found"
+      raise IncorrectInput, "Constant #{str} not found"
     end
 
     # Returns a type name suitable for displaying, for instance, in
