@@ -12,10 +12,13 @@
 # GNU General Public License for more details (in the COPYING file).
 
 require 'CTioga/utils'
+require 'CTioga/boundaries'
 require 'CTioga/elements'
 
 module CTioga
 
+  Version::register_svn_info('$Revision: 760 $', '$Date: 2008-03-02 02:32:05 +0100 (Sun, 02 Mar 2008) $')
+
   # This structure holds the different properties to be transmitted to
   # a Curve object for appropriate drawing. 
 
@@ -215,6 +218,7 @@ EOE
           t.stroke_line(0.0, 0.5, 1.0, 0.5)
         end
         if marker
+          t.line_type = "none"
           t.show_marker( 'x' => 0.5,
                          'y' => 0.5,
                          'marker' => marker,
@@ -303,10 +307,10 @@ EOE
       after = @function.point(index + 1)
       raise "Point invalid" unless point
       tangent = Dvector[0,0]
-      if before
+      if index > 0
         tangent += (point - before)
       end
-      if after
+      if index < (@function.size - 1)
         tangent += (after - point)
       end
       return tangent
@@ -349,12 +353,15 @@ EOE
     # path, not attempt to create a new path or empty what was done
     # before.
     def make_path(t)
+      bnds = Utils::Boundaries.new(parent.effective_bounds)
       if @style.interpolate
         for f in @function.split_monotonic
+          new_f = f.bound_values(*bnds.real_bounds)
           t.append_interpolant_to_path(f.make_interpolant)
         end
       else
-        t.append_points_to_path(@function.x, @function.y)
+        f = @function.bound_values(*bnds.real_bounds)
+        t.append_points_to_path(f.x, f.y)
       end
     end
 
@@ -378,6 +385,7 @@ EOE
       ys = @function.y
 
       if @style.marker
+        t.line_type = [[], 0]   # Always solid for striking markers
         t.stroke_transparency = @style.marker_transparency || 0
         t.fill_transparency = @style.marker_transparency || 0
         t.show_marker('Xs' => xs, 'Ys' => ys,
@@ -443,8 +451,7 @@ EOE
     end
 
     def plot(t = nil)
-      debug "Plotting curve #{identify(self)}"
-      debug "Style #{self.style}"
+      debug "Plotting curve #{inspect}"
       t.context do
 
         # The fill is always first
