@@ -14,11 +14,13 @@
 require 'Dobjects/Dvector'
 require 'CTioga/debug'
 require 'CTioga/log'
-require 'CTioga/axes'
+require 'CTioga/plot_style'
 require 'CTioga/layout'
 
 module CTioga
 
+  Version::register_svn_info('$Revision: 791 $', '$Date: 2008-04-28 14:28:58 +0200 (Mon, 28 Apr 2008) $')
+
   # The base class for every single object that has to appear at one
   # point as a command in the tioga plot. It provides only one function:
   # do(f), which will be used by subclasses to do what they need.
@@ -45,6 +47,24 @@ module CTioga
     def need_style?
       return false
     end
+
+    # We plot everything but parent. If a prefix is given, it is prepended
+    # to all lines but the first (for indentation)
+    def inspect(prefix="")
+      ret = "#<#{self.class.name}:\n"
+      for i in instance_variables
+        next if i == "@parent"
+        var = instance_variable_get(i)
+        ret += "#{prefix}  - #{i} -> "
+        if var.is_a? TiogaElement
+          ret += "#{var.inspect("#{prefix}  ")}\n"
+        else
+          ret += "#{var.inspect}\n"
+        end
+      end
+      ret += "#{prefix}>"
+      return ret
+    end
   end 
   
 
@@ -72,7 +92,7 @@ module CTioga
     attr_reader :elements, :funcalls
 
     # A rescale attribute
-    attr_accessor  :rescale
+    attr_accessor :rescale
 
     # Whether or not to show legend for the current plot
     attr_accessor :show_legend
@@ -229,8 +249,11 @@ module CTioga
 
   class SubPlot < Container
     
-    # Various textual objects laying around:
-    attr_accessor :title, :xlabel, :ylabel
+#     # Various textual objects laying around:
+#     attr_accessor :title, :xlabel, :ylabel
+
+#     # Ticks
+#     attr_accessor :xticks, :yticks
 
     # various important things:
     attr_accessor :legend_specs
@@ -244,9 +267,13 @@ module CTioga
     # the box. It is an array [left,right,top,bottom].
     attr_accessor :plot_margins
 
+    # A PlotSyle object carrying information about the style of the plot,
+    # such as titles/legends/background color...
+    attr_accessor :plot_style
 
-    # The appearance of the edges
-    attr_accessor :edges
+
+#     # The appearance of the edgesax
+#     attr_accessor :edges
 
     def number
       return @elements.length
@@ -273,14 +300,17 @@ module CTioga
         @accept_legend = false
       end
 
-      @title = Label.new(:title)
-      @title.label = "A nice plot" unless parent
+#       @title = Label.new(:title)
+#       @title.label = "A nice plot" unless parent
 
-      @xlabel = Label.new(:xlabel)
-      @xlabel.label = "$x$" unless parent
+#       @xlabel = Label.new(:xlabel)
+#       @xlabel.label = "$x$" unless parent
 
-      @ylabel = Label.new(:ylabel)
-      @ylabel.label = "$y$" unless parent
+#       @ylabel = Label.new(:ylabel)
+#       @ylabel.label = "$y$" unless parent
+
+#       @xticks = TickLabels.new(:xaxis_numeric_label)
+#       @yticks = TickLabels.new(:yaxis_numeric_label)
 
 
       # the hash used by show_plot_with_legend: we inherit it from the parent
@@ -295,7 +325,9 @@ module CTioga
       @bounds = [nil,nil,nil,nil]
       @effective_bounds = nil
 
-      @edges = EdgesAndAxes.new
+#      @edges = EdgesAndAxes.new(@xticks, @yticks)
+
+      @plot_style = PlotStyle.new(self, parent ? true : false)
     end
 
     # Returns the inner frames of the object in terms of absolute
@@ -309,9 +341,10 @@ module CTioga
       return frame
     end
 
-    # The outer frame is the one in which the current element has
-    # all latitudes to do something. It is not a clip box, as the element
-    # might draw outside, but it is the place somehow allocated to it.
+    # The outer frame is the reference frame for the object. It is
+    # either the root frame (the whole plot) or the inner frame of the
+    # parent object. Objects should refrain from writing anything
+    # outside this outer frame.
     def outer_frame(t)
       # The outer frame is either  the root frame
       # or the inner_frame of the parent.
@@ -336,6 +369,8 @@ module CTioga
 
     # Used by children to notify that an axis shoudn't be used
     # anymore.
+    #
+    # TODO: move this to plot_style...
     def disable_axis(axis)
       case axis
       when :x
@@ -364,12 +399,20 @@ module CTioga
       bounds[3] -= height * @plot_margins[3]
 
       # overriding with user-specified values if applicable
+      nb_nans = 0
       4.times do |i|
         if @bounds[i] 
           bounds[i] = @bounds[i] 
         end
         # Replace NaNs with 0
-        bounds[i] = 0 if bounds[i].nan?
+        if bounds[i].nan?
+          bounds[i] = 0 
+          nb_nans += 1
+        end
+      end
+      # In the case all boundaries are NaN, we provide a 0->1 mapping
+      if nb_nans == 4
+        return [0,1,1,0]
       end
       return bounds
     end
@@ -405,8 +448,11 @@ module CTioga
             debug "Reference frame (#{self.class}): #{Utils::frames_str(t)}"
             t.show_plot_with_legend(legend_specs) {
               @effective_bounds = compute_boundaries
+              debug "Plot boundaries: #{@effective_bounds.inspect}"
               debug "Box frame (#{self.class}): #{Utils::frames_str(t)}"
-              @edges.setup(t)
+
+              @plot_style.show_edges(t, self)
+
               t.show_plot(@effective_bounds) {
                 # output frame debug information./
                 debug "Plot frame (#{self.class}): #{Utils::frames_str(t)}"
@@ -415,11 +461,10 @@ module CTioga
                 end
                 
                 debug "Inner frame: #{Utils::framespec_str(inner_frame(t))}"
-                # Show labels
-                for l in [@title, @xlabel, @ylabel]
-                  l.show(t)
-                  debug "Extension -> #{l.extension(t).inspect}"
-                end
+
+                @plot_style.show_background(t, self)
+                @plot_style.show_labels(t, self)
+
                 @elements.each do |e|
                   e.do(t)
                 end
@@ -432,6 +477,14 @@ module CTioga
                   t.save_legend_info(l)
                 end
               } 
+              
+              # Now, additional axes, if applicable
+              t.context do 
+                t.set_bounds(@effective_bounds)
+                # Call the stuff for additional axes.
+                @plot_style.edges.show_additional_axes(t, self)
+              end
+
             }
           }
         }
