@@ -34,6 +34,8 @@ end
 
 module CTioga
 
+  Version::register_svn_info('$Revision: 774 $', '$Date: 2008-03-05 11:43:20 +0100 (Wed, 05 Mar 2008) $')
+
   # A class which represents a dimension, either absolute or
   # relative.
   class Dimension
@@ -278,12 +280,13 @@ module CTioga
     # Returns the extension of the layout
     def extension(t)
       extensions = [0,0,0,0]
+      # TODO: maybe this code should move within the PlotStyle object
       for a in [:title, :xlabel, :ylabel]
         update_extension_with_object(extensions, 
-                                     @object.send(a),
+                                     @object.plot_style.send(a),
                                      t, @object.rescale || 1)
       end
-      update_extension_with_object(extensions, @object.edges,t)
+      update_extension_with_object(extensions, @object.plot_style.edges,t)
       debug "Layout #{"%x" % self.object_id} extensions #{extensions.inspect}"
       return extensions
     end
@@ -383,6 +386,16 @@ module CTioga
   # not get positionned at all in the same way.
   class SimpleLayout < Layout
 
+    # A hash setting the 'bounding box' for the plot. It is relative
+    # to the Container#outer_frame. Defaults to 0: the object occupies
+    # all the frame.
+    attr_accessor :bounding_box
+
+    def initialize(ro, ch = [], children = [] )
+      super
+      @bounding_box = [0.0, 0.0, 0.0, 0.0]
+    end
+
     # Returns the object's frame.
     def legend_specs(t = nil)
       a = extension(t)
@@ -405,8 +418,8 @@ module CTioga
       
       specs = {}
       i = 0
-      for s in %w(left right top bottom)
-        specs["plot_#{s}_margin"] = padding[i]
+      for side in %w(left right top bottom)
+        specs["plot_#{side}_margin"] = padding[i] + @bounding_box[i]
         i += 1
       end
       if @object.display_legend?
