@@ -17,21 +17,28 @@ require 'Tioga/FigureConstants'
 
 module CTioga
 
+  Version::register_svn_info('$Revision: 796 $', '$Date: 2008-06-06 13:49:26 +0200 (Fri, 06 Jun 2008) $')
 
   # A helper function that converts user input into a Tioga
   # color.
   def self.get_tioga_color(str)
-    if str =~ /([\d.]+),([\d.]+),([\d.]+)/
+    if str =~ /([\d.]+),([\d.]+),([\d.]+)/ # Array notation
       color = Dvector.new([$1.to_f,$2.to_f,$3.to_f])
       if color.max > 1
         color *= 1.0/255.0
       end
       return color.to_a
+    elsif str =~ /#([0-9a-fA-F]{6})/ # HTML notation
+      return $1.scan(/../).map { |i| i.to_i(16)/255.0 }
     else
       return Tioga::FigureConstants.const_get(str) 
     end
   end
 
+  # This class is just a means to make an instance of Array stand out
+  class SpecialArray < Array
+  end
+
 
   # The MovingArray class provides a way to scroll through an array,
   # such as a set of colors, markers, linstyles, and so on. The class
@@ -91,8 +98,11 @@ module CTioga
         end
         @current_set = array
       else
-        # If a set is not found, take its name for its unique value
-        @current_set = [name]
+        if name.is_a? SpecialArray
+          @current_set = name
+        else
+          @current_set = [name]
+        end
       end
     end
 
