@@ -1,5 +1,5 @@
-# plotmaker.rb, copyright (c) 2006 by Vincent Fourmond: 
-# The main class for making plots
+# plotmaker.rb: The main class for making plots
+# copyright (c) 2006, 2007, 2008 by Vincent Fourmond
   
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -49,7 +49,9 @@ require 'MetaBuilder/metabuilder'
 
 
 module CTioga
-  
+
+  Version::register_svn_info('$Revision: 790 $', '$Date: 2008-04-28 14:28:22 +0200 (Mon, 28 Apr 2008) $')
+
 
 
   # The regular expression saying that what we see on the command-line
@@ -66,7 +68,6 @@ module CTioga
 
   # A small function to help quoting a string for inclusion
   # in a pdfTeX primitive.
-
   def self.pdftex_quote_string(str)
     return str.gsub(/([%#])|([()])|([{}~_^])|\\/) do 
       if $1
@@ -112,6 +113,8 @@ EOD
 
     # Support for Themes:
     include Themes
+
+    # Axes, edges, labels, tick labels:
     include Axes
 
 
@@ -159,8 +162,6 @@ EOD
       # Initialize the backend structure:
       init_backend_structure
 
-      # Initialize the debugging options:
-#      init_debug
       init_axes
 
       # now, the elements for the structure of the plot:
@@ -267,6 +268,25 @@ EOD
       self.current_object = object
     end
 
+    # Goes out from a child object to its parent. Returns the child
+    # object. Issues a warning if already at top level (in which case
+    # it returns the current object)
+    def leave_child_object
+      prev = current_object
+      if current_object.parent
+        self.current_object = current_object.parent
+      else
+        warn "--end while in top level"
+      end
+      return prev
+    end
+
+
+    # Returns the current object's plot style
+    def current_plot_style
+      return current_object.plot_style
+    end
+
 
 
     # This function reads a configuration file and executes it's
@@ -398,16 +418,6 @@ EOD
       @frame_sides = [val[0], 1.0 - val[1], 1.0 - val[3], val[2]]
     end
 
-    # A function that transforms an inset specification into
-    # margins.
-    def inset_margins(spec)
-      spec =~ /(.*),(.*):([^x]*)(?:x(.*))?/
-      x = $1.to_f; y = $2.to_f; w = $3.to_f
-      h = ($4 || $3).to_f
-      margins = [x - w/2, 1 - (x + w/2), 1 - (y+h/2), y - h/2]
-      return margins
-    end
-
     # This function prepares the parser by giving it the arguments and
     # some small help text going along with them. This function will
     # unfortunately get really large with time, but there's nothing
@@ -428,11 +438,11 @@ EOD
 
       @parser.separator "\nGlobal look:"
 
-      @parser.on("--background COLOR",
+      @parser.on("--[no-]background [COLOR]",
                  "Sets the background color for plots") do |c|
-        color = CTioga.get_tioga_color(c)
-        @current_object.add_elem(TiogaFuncall.new(:fill_color=, color))
-        @current_object.add_elem(TiogaFuncall.new(:fill_frame))
+        # Keep it to false if only false
+        c = CTioga.get_tioga_color(c) if c
+        current_plot_style.background_color = c
       end
 
       @parser.on("--aspect-ratio [RATIO]",
@@ -480,13 +490,6 @@ EOD
       axes_options(@parser)
 
       @parser.separator "\nSubfigures, subplots, regions..."
-#       @parser.on("--subplot",
-#                  "Start a subplot") do
-#         plot = SubPlot.new(:subplot, current_object)
-#         @current_object.add_elem(plot)
-#         self.current_object = plot
-#       end
-
 
       @parser.on("--y2",
                  "Switch to an alternative Y axis") do
@@ -499,10 +502,10 @@ EOD
         end
         plot = SharedAxisPlot.new(:y, current_object)
         # The label will be on the right
-        plot.ylabel.side = RIGHT
-        plot.edges.yaxis.loc = RIGHT
-        plot.edges.set_edges(AXIS_HIDDEN, :left,:top,:bottom)
-        current_object.edges.set_edges(AXIS_LINE_ONLY, :right)
+        plot.plot_style.ylabel.side = RIGHT
+        plot.plot_style.edges.yaxis.loc = RIGHT
+        plot.plot_style.edges.set_edges(AXIS_HIDDEN, :left,:top,:bottom)
+        current_plot_style.edges.set_edges(AXIS_LINE_ONLY, :right)
         # A simple layout will control the plot
         layout = FixedLayout.new(plot)        
         # And this layout should report its info to
@@ -510,7 +513,11 @@ EOD
         #
         # Note that we add the child's *layout* to the
         # objects layout, not the child itself.
-        current_object.layout.add_child(layout)
+        #
+        # Won't fail if the current plot does not have a layout
+        if current_object.layout
+          current_object.layout.add_child(layout)
+        end
         enter_child_object(plot)
       end
 
@@ -520,7 +527,7 @@ EOD
           current_object.disable_axis(:y)
         rescue
           # Purely ignore errors
-          warn "Something should have happened here "+
+          warn "Something should have happened here " +
             "that did not happen properly"
         end
         plot = SharedAxisPlot.new(:x, current_object)
@@ -539,19 +546,13 @@ EOD
         enter_child_object(plot)
       end
 
-      @parser.on("--inset SPECS",
+      @parser.on("--inset SPEC",
                  "Creates an inset with the given",
-                 "specifications: " +
-                 "x,y:w[xh]",
-                 "(x,y: position of the center, w width and",
-                 "h height if different than width).", 
-                 "All numbers " +
-                 "are between 0 and 1" ) do |a|
+                 "specifications (x,y:w[xh] or x1,y1;x2,y2)" ) do |a|
         plot = SubPlot.new(:subplot, current_object)
         plot.show_legend = false
-        @current_object.add_elem(plot)
-        self.current_object = plot
-        margins = inset_margins(a)
+        enter_child_object(plot)
+        margins = Utils::inset_margins(a)
         debug "inset margins: #{margins.inspect}"
         # With force_position, there is no need for a layout.
         current_object.force_position = margins
@@ -559,7 +560,7 @@ EOD
         current_object.accept_legend = false
       end
 
-      @parser.on("--zoom-inset SPECS",
+      @parser.on("--zoom-inset SPEC",
                  "Creates an inset as with --inset " +
                  "and copies all the ",
                  "elements in the current plot there") do |a|
@@ -576,13 +577,72 @@ EOD
         end
         plot.disable_legend = false
         # Better add it afterwards...
-        @current_object.add_elem(plot)
-        self.current_object = plot
-        margins = inset_margins(a)
+        enter_child_object(plot)
+        margins = Utils::inset_margins(a)
         debug "zoom margins: #{margins.inspect}"
         current_object.force_position = margins
       end
 
+      @parser.on("--next-inset SPEC",
+                 "Equivalent to --end --inset SPEC, except that",
+                 "various style information are carried from",
+                 "the previous inset") do |a|
+        plot = SubPlot.new(:subplot, current_object)
+        plot.show_legend = false
+        old = leave_child_object
+        enter_child_object(plot)
+
+        # Force copy of the style:
+        plot.plot_style = old.plot_style.deep_copy(plot)
+
+        margins = Utils::inset_margins(a)
+        debug "inset margins: #{margins.inspect}"
+        # With force_position, there is no need for a layout.
+        current_object.force_position = margins
+        # We redirect all legends for the inset to the parent.
+        current_object.accept_legend = false
+      end
+
+      @parser.on("--subplot SPEC",
+                 "Creates a subplot with the given specifications",
+                 "(x,y:w[xh] or x1,y1;x2,y2)" ) do |a|
+        plot = SubPlot.new(:subplot, current_object)
+        enter_child_object(plot)
+        margins = Utils::inset_margins(a)
+
+        debug "subplot margins: #{margins.inspect}"
+        # In contrast with --inset, we use a layout:
+        current_object.layout = SimpleLayout.new(current_object)
+        current_object.layout.bounding_box = margins
+
+        # We do not redirect legends for a subplot
+        current_object.accept_legend = true
+
+      end
+
+      @parser.on("--next-subplot SPEC",
+                 "Creates a subplot with the given specifications",
+                 "(x,y:w[xh] or x1,y1;x2,y2)" ) do |a|
+        plot = SubPlot.new(:subplot, current_object)
+        old = leave_child_object
+        enter_child_object(plot)
+        margins = Utils::inset_margins(a)
+
+        # Force copy of the style:
+        plot.plot_style = old.plot_style.deep_copy(plot)
+
+        debug "subplot margins: #{margins.inspect}"
+        # In contrast with --inset, we use a layout:
+        current_object.layout = SimpleLayout.new(current_object)
+        current_object.layout.bounding_box = margins
+
+        # We do not redirect legends for a subplot
+        current_object.accept_legend = true
+
+      end
+
+
+
       @parser.on("--disable-legends",
                  "Disable the display of legends for the current " +
                  "subplot/subfigure") do
@@ -604,8 +664,8 @@ EOD
         if which == :columns= or which == :rows=
           current_object.layout = current_object.layout.
             convert_layout(GridLayout)
-          current_object.edges.disable_axis_display(:x, AXIS_HIDDEN)
-          current_object.edges.disable_axis_display(:y, AXIS_HIDDEN)
+
+          current_plot_style.hide_axis_and_edges(:x, :y)
 
           # We rescale the padding by a factor of 1/number, so it looks
           # reasonable in the end.
@@ -616,9 +676,8 @@ EOD
           # We use the default padding for the chidren
           use_default_padding
 
-          current_object.xlabel.label = nil
-          current_object.ylabel.label = nil
-          # We make two columns, just for fun !
+          current_plot_style.set_xy_labels(nil, nil)
+
           current_object.layout.send(which, number)
           plot = SubPlot.new(:subplot, current_object)
           plot.accept_legend = false
@@ -644,20 +703,23 @@ EOD
         current_object.layout = current_object.layout.
           convert_layout(GridLayout)
 #        current_object.edges.disable
-        current_object.edges.disable_axis_display(:x, AXIS_HIDDEN)
-        current_object.edges.disable_axis_display(:y, AXIS_HIDDEN)
+
+        current_plot_style.hide_axis_and_edges(:x, :y)
+
         current_object.layout_preferences.padding[2] = Dimension.new(0.0)
         current_object.layout_preferences.padding[3] = Dimension.new(0.0)
-        # We make two columns, just for fun !
+
         current_object.layout.columns = 1
         plot = SharedAxisPlot.new(:y, current_object)
         @next_spec = [SharedAxisPlot, :y]
         @current_object.layout.add_child(GridItemLayout.new(plot))
         @current_object.add_elem(plot)
-        plot.xlabel.label = current_object.xlabel.label
-        current_object.xlabel.label = nil
-        plot.title.label = current_object.title.label
-        current_object.title.label = nil
+        
+        # We copy the X label and the title
+        plot.plot_style.xlabel.label = current_plot_style.xlabel.label
+        current_plot_style.xlabel.label = nil
+        plot.plot_style.title.label = current_plot_style.title.label
+        current_plot_style.title.label = nil
         self.current_object = plot
 
         # We cancel vertical padding by default:
@@ -668,10 +730,12 @@ EOD
         # Forwards the x label to the child, and cancels it here.
         # The code to be run on the old and new tings
         @next_block = proc do |old, new|
-          old.edges.disable_axis_display(:x)
-          new.xlabel.label = old.xlabel.label if old.xlabel.label
-          old.xlabel.label = nil
-          new.ylabel.label = old.ylabel.label if old.ylabel.label
+          old.plot_style.edges.
+            set_axis_and_edges_style(:x, AXIS_WITH_TICKS_ONLY)
+          old.plot_style.xlabel.label = nil
+
+          # Disable title by default
+          new.plot_style.title.label = nil
           use_default_padding(new)
         end
       end
@@ -776,11 +840,7 @@ EOD
 
       @parser.on("--end",
                  "Ends the last subplot or region") do 
-        if current_object.parent
-          self.current_object = current_object.parent
-        else
-          warn "--end while in top level"
-        end
+        leave_child_object
       end
       
 
@@ -822,6 +882,12 @@ EOD
         @tex_fontsize = size
       end
 
+      ## Legend scale
+      @parser.on("--legend-scale FACT",
+                 "Sets the scale for the legends") do |l|
+        add_elem_funcall(:legend_scale=, Float(l))
+      end
+
       @parser.on("--tick-label-scale SCALE",
                  "Sets the scale of the text for tick labels") do |l|
         add_elem_funcall(:xaxis_numeric_label_scale=,Float(l))
@@ -854,7 +920,7 @@ EOD
       @parser.on("--legend-inside SPEC", 
                  "Makes a legend inside the plot.",
                  "See --inset for the format of SPEC") do |a|
-        margins = inset_margins(a)
+        margins = Utils::inset_margins(a)
         specs = margins.to_frame("legend_%s_margin")
         debug "Specs: #{specs.inspect}"
         current_object.layout_preferences.legend_position = :inside
@@ -865,7 +931,7 @@ EOD
       @parser.separator "\nGraphic primitives:"
       @parser.on("-d","--draw SPEC",
                  "Draw a graphic primitive") do |spec|
-        add_elems_to_current(TiogaPrimitiveMaker.parse_spec(spec, self))
+        add_elems_to_current(*TiogaPrimitiveMaker.parse_spec(spec, self))
       end
       @parser.on("--draw-help",
                  "Display what is known about graphic", 
@@ -933,10 +999,14 @@ EOD
         @fig_name = name
       end
 
+      @parser.on("--echo", "Prints command-line to standard output") do
+        puts "Command-line used:"
+        puts @command_line
+      end
+
       @parser.on("--display-commandline",
                  "Adds the command line used to make",
-                 "to graph on the " +
-                 "bottom of the graph") do 
+                 "to graph on the bottom of the graph") do 
         add_elem_funcall(:show_marker, 
                          {
                            "x" => 0.5,
@@ -1006,18 +1076,21 @@ EOD
 
     # Start the next object in a complex plot (--grid, --col...)
     def next_object
-        old = current_object
-        self.current_object = current_object.parent
-        a = @next_spec.dup      # We need to dup, else shift would make the
-        # next curve not that happy...
-        cls = a.shift
-        a <<  current_object
-        plot = cls.new(*a)
-        @current_object.layout.add_child(GridItemLayout.new(plot))
-        @current_object.add_elem(plot)
-        self.current_object = plot
+      # We swicth back to the parent object
+      old = leave_child_object
+      a = @next_spec.dup      # We need to dup, else shift would make the
+      # next curve not that happy...
+      cls = a.shift
+      # We push the parent object as the last element for new.
+      a << current_object
+      plot = cls.new(*a)
+      @current_object.layout.add_child(GridItemLayout.new(plot))
+      enter_child_object(plot)
+
+      # We copy the style from the old plot to the new one:
+      plot.plot_style = old.plot_style.deep_copy(plot)
         
-        @next_block.call(old, plot) if @next_block
+      @next_block.call(old, plot) if @next_block
     end
 
 
@@ -1066,6 +1139,10 @@ EOD
       data.mul!(:x,x_factor) if x_factor
       data.mul!(:y,y_factor) if y_factor
 
+      # Add an offset
+      data.add!(:x,x_offset) if x_offset
+      data.add!(:y,y_offset) if y_offset
+
       # Implement a logarithmic scale
       data.safe_log10!(:x) if x_log
       data.safe_log10!(:y) if y_log
@@ -1125,24 +1202,56 @@ EOD
       end
 
       first = true
-      begin
-        @parser.order!(@args) do |spec|
-          # We use Backend#expand to leave room for the backend to
-          # interpret the text as many different sets.
-          sets = expand_spec(spec)
-          info "Expanding #{spec} to #{sets.join(', ')}"
-          # Auto --next: 
-          next_object if (@auto_next == :element and !first)
-          for set in sets
-            next_object if (@auto_next == :dataset and !first)
-            process_data_set(set)
-            first = false
+      done = false
+      while ! done
+        begin
+          @parser.order!(@args) do |spec|
+            # We use Backend#expand to leave room for the backend to
+            # interpret the text as many different sets.
+            sets = expand_spec(spec)
+            info "Expanding #{spec} to #{sets.join(', ')}"
+            # Auto --next: 
+            next_object if (@auto_next == :element and !first)
+            for set in sets
+              next_object if (@auto_next == :dataset and !first)
+              process_data_set(set)
+              first = false
+            end
           end
+          done = true
+        rescue OptionParser::InvalidOption => o
+          # Here, we should try to be clever about options:
+          new_arg = nil
+          option_name = o.args.first.gsub(/^--?/,'')
+          # First, we check whether it corresponds to a shortcut:
+          if Shortcut.has? option_name
+            new_arg = ["--short", option_name]
+          # Or an option of the current backend:
+          elsif backend.description.param_hash.key? option_name
+            new_arg = ["--#{backend.description.name}-#{option_name}"]
+          end
+
+          if new_arg
+            info "Option #{o.args.first} was reinterpreted " +
+              "as #{new_arg.join ' '}"
+            @args.unshift(*new_arg)
+          else
+            debug "Invalid option: #{o.inspect}"
+            debug "Remaining args: #{@args.inspect}"
+            error "Invalid option: #{o.args.first}. Please run " + 
+              "ctioga --help to have a list of valid ones"
+            exit
+          end
+        rescue Exception => ex
+          # Do not catch normal exit !
+          if ex.is_a? SystemExit
+            raise
+          end
+          fatal "ctioga encountered the following problem: #{ex.to_s}"
+          debug "Error backtrace: #{ex.backtrace.join "\n"}"
+          fatal "Aborting"
+          exit
         end
-      rescue OptionParser::InvalidOption => o
-        error "Invalid option: #{o.args.first}. Please run " + 
-          "ctioga --help to have a list of valid ones"
-        exit
       end
     end
 
@@ -1158,6 +1267,8 @@ EOD
         t.tex_preview_preamble += 
           "\n\\pdfinfo {\n  /Title (" +
           CTioga.pdftex_quote_string(@command_line) +
+          ")\n" + "/Creator(" +
+          CTioga.pdftex_quote_string("ctioga #{Version::version}") +
           ")\n}\n"
       end
 
@@ -1212,8 +1323,10 @@ EOD
     # Ctioga's banner
     def banner
       <<"EOBANNER" 
-This is ctioga version #{CTioga.version}, copyright (C) 2006, 2007 by
-Vincent Fourmond ctioga comes with absolutely NO WARRANTY. This is
+This is ctioga version #{Version::version}
+ctioga is copyright (C) 2006, 2007 by Vincent Fourmond 
+
+ctioga comes with absolutely NO WARRANTY. This is
 free software, you are welcome to redistribute it under certain
 conditions. See the COPYING file in the original tarball for
 details. You are also welcome to contribute if you like it, see in
@@ -1399,7 +1512,7 @@ EOBANNER
   # to go through shell expansion. For now, it won't work with
   # strings containing ', but that's already a good help.
   def CTioga.shell_quote_string(str)
-    if str =~ /[\s"*$()\[\]{}']/
+    if str =~ /[\s"*$()\[\]{}';]/
       if str =~ /'/
         a = str.gsub(/(["$\\])/) { "\\#$1" }
         return "\"#{a}\""
