@@ -18,6 +18,8 @@ require 'CTioga/shortcuts'
 
 module CTioga
 
+  Version::register_svn_info('$Revision: 796 $', '$Date: 2008-06-06 13:49:26 +0200 (Fri, 06 Jun 2008) $')
+
   # The namespace for themes. Supposedly, all themes should end up
   # here.
   module Themes
@@ -64,39 +66,6 @@ module CTioga
 
     end
 
-    # Some constants that could be of interest to themes:
-    module Data
-      include Tioga::FigureConstants
-
-      COLORS = { 
-        "standard" => 
-        [Red, Green, Blue, Cyan, Magenta, Orange],
-        "pastel1" => 
-        [MediumSeaGreen, RoyalBlue, Pumpkin, DarkChocolate, Lilac, Crimson],
-        "colorblind" => 
-        [BrightBlue, Goldenrod, Coral, Lilac, FireBrick, RoyalPurple],
-      }
-      
-      # The sets of markers. Well, sets, that's a big word...
-      MARKERS = { 
-        "standard" => 
-        [Bullet, TriangleUp, Square, Plus, Times],
-      }
-
-      Line_Type_Dash_Dot_Dot = [[5,2,1,2,1,2],0]
-      
-      # Linestyles.
-      LINES = {
-        "standard" => 
-        [Line_Type_Solid, 
-         Line_Type_Dots, 
-         Line_Type_Dashes, 
-         Line_Type_Dot_Long_Dash,
-         Line_Type_Dash_Dot_Dot
-        ],
-      }
-      
-    end
 
     # Now, some code to be used as an inclusion directly.
 
@@ -130,8 +99,8 @@ module CTioga
       @saved_overrides = {}
       
 
-      # For now, manually require themes. This *is* ugly ;-) !
-#      require 'CTioga/themes/classical'
+      # We look for themes in the themes subdirectory of the directory
+      # where this file is found and in the $HOME/.ctioga/themes
       for f in Dir.glob(File.dirname(__FILE__) + "/themes/*") +
           Dir.glob("#{ENV['HOME']}/.ctioga/themes/*")
         begin
@@ -168,7 +137,24 @@ module CTioga
     DisableRe = /no|off/i
 
     # Processes a style argument.
-    def style_argument(style_element, value, namespace)
+
+    # A correspondance style_element -> style type
+    StyleTypes = {
+      :color => :color,
+      :marker => :marker,
+      :marker_color => :color,
+      :line_style => :line_style,
+      :linewidth => :float,
+      :error_bar_color => :color,
+      :marker_scale => :float,
+    }
+
+    # Parses a style argument, in three different ways:
+    # * an 'auto' will remove the override
+    # * something matching DisableRe will turn it false
+    # * anything else will be converted using the appropriate
+    #   type found in StyleTypes or using the block given
+    def style_argument(style_element, value)
       if value =~ /auto/i
         override_style.delete(style_element)
         debug "Deleting override #{style_element}"
@@ -176,15 +162,23 @@ module CTioga
       elsif value =~ DisableRe or not value
         override_style[style_element] = false
       else
-        override_style[style_element] = if block_given?
-                                          yield(value)
-                                        else
-                                          begin
-                                            namespace.const_get(value)
-                                          rescue
-                                            value
-                                          end
-                                        end
+        override_style[style_element] = 
+          if block_given?
+            yield(value)
+          else
+            begin
+              MetaBuilder::ParameterType.from_string(StyleTypes[style_element],
+                                                     value)
+            rescue Exception => e
+              # TODO: ignore the parameter when the
+              if e.is_a? MetaBuilder::ParameterType::IncorrectInput
+                warn "The argument for #{style_element} was not recognized:"
+                warn e.message
+                warn "Expect problems later"
+              end
+              value
+            end
+          end
       end
       debug "Setting override #{style_element} to " +
         "#{override_style[style_element].inspect}"
@@ -197,33 +191,29 @@ module CTioga
                 "Sets the color for drawing the curves.", 
                 "Use 'auto' to leave the decision to ",
                 "the themes, and 'no' to get no lines.") do |val|
-        style_argument(:color, val, nil) do |v|
-          CTioga.get_tioga_color(v)
-        end
+        style_argument(:color, val)
       end
 
       parser.on("-m", "--[no-]marker [MARKER]",
                  "Sets the markers for drawing data points", 
                  "Use 'auto' to get automatic markers,",
                  "and 'no' to get no marker (the default)") do |val|
-        style_argument(:marker, val, Tioga::FigureConstants)
+        style_argument(:marker, val)
       end
       
       parser.on("--marker-color COLOR",
                  "Sets the markers' color. See also --color") do |val|
-        style_argument(:marker_color, val, nil) do |v|
-          CTioga.get_tioga_color(v)
-        end
+        style_argument(:marker_color, val)
       end
 
       parser.on("--line-style STYLE",
                 "Sets the line style") do |val|
-        style_argument(:line_style, val, Data)
+        style_argument(:line_style, val)
       end
 
       parser.on("--line-width WIDTH",
                 "Sets the line width") do |w|
-        style_argument(:linewidth, w, nil) do |val|
+        style_argument(:linewidth, w) do |val|
           Float(val)
         end
       end
@@ -238,22 +228,18 @@ module CTioga
                 "The scale of the markers used for curves.", 
                 "Defaults to 0.5"
                  ) do |w|
-        style_argument(:marker_scale, w, nil) do |v|
-          Float(v)
-        end
+        style_argument(:marker_scale, w)
       end
 
       parser.on("--error-bar-color COLOR",
                 "Sets the error bars' color. See also --color") do |val|
-        style_argument(:error_bar_color, val, nil) do |v|
-          CTioga.get_tioga_color(v)
-        end
+        style_argument(:error_bar_color, val)
       end
 
       parser.on("--drawing-order ORDER",
                 "Sets the order for drawing curve elements") do |val|
         begin
-          style_argument(:drawing_order, val, nil) do |v|
+          style_argument(:drawing_order, val) do |v|
             i = Integer(v)
             CurveStyle::DrawingOrder.fetch(i) # To raise an
             # exception in case i is not valid
@@ -271,7 +257,7 @@ module CTioga
       parser.separator 'Transparency options'
       parser.on("--transparency T",
                 "Sets the transparency for lines.") do |val|
-        style_argument(:transparency, val, nil) do |v|
+        style_argument(:transparency, val) do |v|
           begin
             Float(v)
           rescue
@@ -282,7 +268,7 @@ module CTioga
 
       parser.on("--marker-transparency T",
                 "Sets the transparency for markers.") do |val|
-        style_argument(:marker_transparency, val, nil) do |v|
+        style_argument(:marker_transparency, val) do |v|
           begin
             Float(v)
           rescue
@@ -293,7 +279,7 @@ module CTioga
 
       parser.on("--error-bar-transparency T",
                 "Sets the transparency for error bars.") do |val|
-        style_argument(:error_bars_transparency, val, nil) do |v|
+        style_argument(:error_bars_transparency, val) do |v|
           begin
             Float(v)
           rescue
@@ -306,21 +292,21 @@ module CTioga
       parser.separator 'Filled curves'
       parser.on("--fill TYPE",
                 "Set the filling type for curves to TYPE") do |val|
-        style_argument(:fill_type, val, nil) do |v|
+        style_argument(:fill_type, val) do |v|
           Utils.interpret_arg(v, CurveStyle::FillTypeArguments)
         end
       end
 
       parser.on("--fill-color COLOR",
                 "Set the fill color") do |val|
-        style_argument(:fill_color, val, nil) do |v|
+        style_argument(:fill_color, val) do |v|
           CTioga.get_tioga_color(v)
         end
       end
 
       parser.on("--fill-transparency T",
                 "Set the fill color") do |val|
-        style_argument(:fill_transparency, val, nil) do |v|
+        style_argument(:fill_transparency, val) do |v|
           begin
             Float(v)
           rescue
@@ -335,7 +321,7 @@ module CTioga
                 "as histograms. Deprecated.",
                 "Please use the --hist option") do |w|
         @histogram = w
-        style_argument(:hist_type, false, nil)
+        style_argument(:hist_type, false)
       end
       parser.on("--hist TYPE",
                 "Makes histograms from the next curves.",
@@ -344,7 +330,7 @@ module CTioga
         w = false if w =~ DisableRe
         if w
           @histogram = true
-          style_argument(:hist_type, w, nil) do |v|
+          style_argument(:hist_type, w) do |v|
             Utils.interpret_arg(v, CurveStyle::FillTypeArguments)
           end
         else 
@@ -361,14 +347,14 @@ module CTioga
                 "The ratio of the width drawn over the ",
                 "total width") do |w|
         a = Float(w)
-        style_argument(:hist_left, (1 - a)/2, nil)
-        style_argument(:hist_right, (1 + a)/2, nil)
+        style_argument(:hist_left, (1 - a)/2)
+        style_argument(:hist_right, (1 + a)/2)
       end
 
       parser.on("--hist-left LEFT",
                 "The position of the left side of the step",
                 "relative to the total step") do |val|
-        style_argument(:hist_left, val, nil) do |v|
+        style_argument(:hist_left, val) do |v|
           begin
             Float(v)
           rescue
@@ -378,7 +364,7 @@ module CTioga
       end
       parser.on("--hist-right RIGHT",
                 "Pendant of --hist-left for the right") do |val|
-        style_argument(:hist_right, val, nil) do |v|
+        style_argument(:hist_right, val) do |v|
           begin
             Float(v)
           rescue
@@ -397,8 +383,7 @@ module CTioga
       end
 
       parser.on("--theme-list",
-                "Lists available themes."
-                 ) do 
+                "Lists available themes.") do 
         puts
         puts "Currently available themes are"
         puts
@@ -406,8 +391,7 @@ module CTioga
       end
 
       parser.on("--reset-theme",
-                "Resets theme defaults."
-                 ) do 
+                "Resets theme defaults.") do 
         choose_theme(@theme_name)
       end
 
@@ -417,10 +401,24 @@ module CTioga
         choose_theme('mono')
       end
 
+      # TODO: there is no reason why sets should be restricted
+      # to the following things. All style things should
+      # potentially have a corresponding style-set.
+      #
+      # In short, all styles things should be converted to something
+      # more in the spirit of Parameter and the like...
+
       # Sets:
       # Note that, due to scoping side-effects, one has to use the
       # block form of the iteration, else type gets overwritten
       # and we end up writing only to the last possibility.
+      #
+      # In addition to (and taking over) specifying named sets,
+      # it is possible to provide:
+      # * a single style element, in which case the set becomes the
+      #   new element
+      # * a |-separated list of elements, in which case the set
+      #   becomes the given elements
       {
         :colors => "color",
         :markers =>  "marker",
@@ -430,13 +428,35 @@ module CTioga
         parser.on("--#{name}-set SET", 
                   "Choose the current set for #{name} ",
                   "(#{@theme.sets[type].available_sets.join(' ')})") do |set|
-          debug "Using set #{set} for #{type} on theme " +
+          debug "Using set '#{set}' for #{type} on theme " +
             "#{current_theme.class}"
+          begin
+            # If there is one '|', we split the 
+            if set =~ /\|/
+              set = SpecialArray.new(set.split(/\|/).map do |x|
+                MetaBuilder::ParameterType.
+                from_string(StyleTypes[name.tr('-','_').to_sym], x)
+              end)
+              info "Setting set #{name} to objects #{set.inspect}"
+            else
+              set = MetaBuilder::ParameterType.
+                from_string(StyleTypes[name.tr('-','_').to_sym], set)
+              info "Setting set #{name} to single-value object #{set.inspect}"
+            end
+          rescue Exception => e
+            debug "Exception raised on set detection code: #{e.inspect}"
+          end
           current_theme.choose_set(type, set)
         end
       end
 
       parser.separator 'Style manipulations'
+
+      parser.on("--skip-style",
+                "Skips the next style" ) do 
+        @theme.next_curve_style("useless")
+      end
+
       parser.on("-s", "--same-style",
                 "Uses the same style as last curve as a base ",
                 "for next curve") do 
