@@ -3,10 +3,9 @@
 # I add some features to ctioga, I don't break everything existing.
 # As such, it also stands as a 'feature list' for ctioga.
 
-export CTIOGA="--xpdf --clean-all --math"
+. ./test_style.sh
 
 # Then, we try various styles:
-
 ctioga -N -t 'Transparency and fills' \
     --fill y-axis --fill-transparency 0.6 'sin(x)' 'cos(x)'
 ctioga -N -t 'Markers' \
@@ -23,3 +22,14 @@ ctioga -N -t 'Override manipulations' \
     --save-override cloud --reset-override \
     '1 - x**2' '10 - x**2' --use-override cloud 'x**2 - 10'
 
+# Now testing background colors
+
+ctioga -N -t 'Turquoise background' \
+    --background PaleTurquoise 'sin(x)**2' 'cos(x)**2'
+
+ctioga -N -t 'Backgrounds with --grid' \
+    --grid column=2 \
+    --background PaleTurquoise 'sin(x)**2' 'cos(x)**2' \
+    --next 'tan(x)' --next --background Chiffon \
+    'x**2' 'x**2 + 10'
+
